@@ -1,16 +1,23 @@
+require 'arel/collectors/sql_string'
+
 module Arel
   class TreeManager
     include Arel::FactoryMethods
 
     attr_reader :ast, :engine
 
+    attr_accessor :bind_values
+
     def initialize engine
       @engine = engine
       @ctx    = nil
+      @bind_values = []
     end
 
     def to_dot
-      Visitors::Dot.new.accept @ast
+      collector = Arel::Collectors::PlainString.new
+      collector = Visitors::Dot.new.accept @ast, collector
+      collector.value
     end
 
     def visitor
@@ -18,7 +25,9 @@ module Arel
     end
 
     def to_sql
-      visitor.accept @ast
+      collector = Arel::Collectors::SQLString.new
+      collector = visitor.accept @ast, collector
+      collector.value
     end
 
     def initialize_copy other
