@@ -6,24 +6,29 @@ module Arel
         super
       end
 
-      def accept node, &block
+      def accept node, collector, &block
         @block = block if block_given?
         super
       end
 
       private
 
-      def visit_Arel_Nodes_Assignment o, a
+      def visit_Arel_Nodes_Assignment o, collector
         if o.right.is_a? Arel::Nodes::BindParam
-          "#{visit o.left, a} = #{visit o.right, a}"
+          collector = visit o.left, collector
+          collector << " = "
+          visit o.right, collector
         else
           super
         end
       end
 
-      def visit_Arel_Nodes_BindParam o, a
+      def visit_Arel_Nodes_BindParam o, collector
         if @block
-          @block.call
+          val = @block.call
+          if String === val
+            collector << val
+          end
         else
           super
         end
