@@ -66,7 +66,7 @@ module Arel
           relation[:id].gt(10).must_be_kind_of Nodes::GreaterThan
         end
 
-        it 'should generate >= in sql' do
+        it 'should generate > in sql' do
           relation = Table.new(:users)
           mgr = relation.project relation[:id]
           mgr.where relation[:id].gt(10)
@@ -74,6 +74,28 @@ module Arel
             SELECT "users"."id" FROM "users" WHERE "users"."id" > 10
           }
         end
+
+        it 'should handle comparing with a subquery' do
+          users = Table.new(:users)
+
+          avg = users.project(users[:karma].average)
+          mgr = users.project(Arel.star).where(users[:karma].gt(avg))
+
+          mgr.to_sql.must_be_like %{
+            SELECT * FROM "users" WHERE "users"."karma" > (SELECT AVG("users"."karma") FROM "users")
+          }
+        end
+
+        it 'should accept various data types.' do
+          relation = Table.new(:users)
+          mgr = relation.project relation[:id]
+          mgr.where relation[:name].gt('fake_name')
+          mgr.to_sql.must_match %{"users"."name" > 'fake_name'}
+
+          current_time = ::Time.now
+          mgr.where relation[:created_at].gt(current_time)
+          mgr.to_sql.must_match %{"users"."created_at" > '#{current_time}'}
+        end
       end
 
       describe '#gt_any' do
@@ -122,6 +144,17 @@ module Arel
             SELECT "users"."id" FROM "users" WHERE "users"."id" >= 10
           }
         end
+
+        it 'should accept various data types.' do
+          relation = Table.new(:users)
+          mgr = relation.project relation[:id]
+          mgr.where relation[:name].gteq('fake_name')
+          mgr.to_sql.must_match %{"users"."name" >= 'fake_name'}
+
+          current_time = ::Time.now
+          mgr.where relation[:created_at].gteq(current_time)
+          mgr.to_sql.must_match %{"users"."created_at" >= '#{current_time}'}
+        end
       end
 
       describe '#gteq_any' do
@@ -170,6 +203,17 @@ module Arel
             SELECT "users"."id" FROM "users" WHERE "users"."id" < 10
           }
         end
+
+        it 'should accept various data types.' do
+          relation = Table.new(:users)
+          mgr = relation.project relation[:id]
+          mgr.where relation[:name].lt('fake_name')
+          mgr.to_sql.must_match %{"users"."name" < 'fake_name'}
+
+          current_time = ::Time.now
+          mgr.where relation[:created_at].lt(current_time)
+          mgr.to_sql.must_match %{"users"."created_at" < '#{current_time}'}
+        end
       end
 
       describe '#lt_any' do
@@ -218,6 +262,17 @@ module Arel
             SELECT "users"."id" FROM "users" WHERE "users"."id" <= 10
           }
         end
+
+        it 'should accept various data types.' do
+          relation = Table.new(:users)
+          mgr = relation.project relation[:id]
+          mgr.where relation[:name].lteq('fake_name')
+          mgr.to_sql.must_match %{"users"."name" <= 'fake_name'}
+
+          current_time = ::Time.now
+          mgr.where relation[:created_at].lteq(current_time)
+          mgr.to_sql.must_match %{"users"."created_at" <= '#{current_time}'}
+        end
       end
 
       describe '#lteq_any' do
@@ -258,12 +313,11 @@ module Arel
           relation[:id].average.must_be_kind_of Nodes::Avg
         end
 
-        # FIXME: backwards compat. Is this really necessary?
-        it 'should set the alias to "avg_id"' do
+        it 'should generate the proper SQL' do
           relation = Table.new(:users)
           mgr = relation.project relation[:id].average
           mgr.to_sql.must_be_like %{
-            SELECT AVG("users"."id") AS avg_id
+            SELECT AVG("users"."id")
             FROM "users"
           }
         end
@@ -275,12 +329,11 @@ module Arel
           relation[:id].maximum.must_be_kind_of Nodes::Max
         end
 
-        # FIXME: backwards compat. Is this really necessary?
-        it 'should set the alias to "max_id"' do
+        it 'should generate the proper SQL' do
           relation = Table.new(:users)
           mgr = relation.project relation[:id].maximum
           mgr.to_sql.must_be_like %{
-            SELECT MAX("users"."id") AS max_id
+            SELECT MAX("users"."id")
             FROM "users"
           }
         end
@@ -299,12 +352,11 @@ module Arel
           relation[:id].sum.must_be_kind_of Nodes::Sum
         end
 
-        # FIXME: backwards compat. Is this really necessary?
-        it 'should set the alias to "sum_id"' do
+        it 'should generate the proper SQL' do
           relation = Table.new(:users)
           mgr = relation.project relation[:id].sum
           mgr.to_sql.must_be_like %{
-            SELECT SUM("users"."id") AS sum_id
+            SELECT SUM("users"."id")
             FROM "users"
           }
         end
@@ -329,7 +381,7 @@ module Arel
           attribute = Attribute.new nil, nil
           equality = attribute.eq 1
           equality.left.must_equal attribute
-          equality.right.must_equal 1
+          equality.right.val.must_equal 1
           equality.must_be_kind_of Nodes::Equality
         end
 
@@ -496,15 +548,109 @@ module Arel
         end
       end
 
-      describe '#in' do
-        it 'can be constructed with a list' do
+      describe 'with a range' do
+        it 'can be constructed with a standard range' do
+          attribute = Attribute.new nil, nil
+          node = attribute.between(1..3)
+
+          node.must_equal Nodes::Between.new(
+            attribute,
+            Nodes::And.new([
+              Nodes::Casted.new(1, attribute),
+              Nodes::Casted.new(3, attribute)
+            ])
+          )
         end
 
-        it 'should return an in node' do
+        it 'can be constructed with a range starting from -Infinity' do
           attribute = Attribute.new nil, nil
-          node = Nodes::In.new attribute, [1,2,3]
-          node.left.must_equal attribute
-          node.right.must_equal [1, 2, 3]
+          node = attribute.between(-::Float::INFINITY..3)
+
+          node.must_equal Nodes::LessThanOrEqual.new(
+            attribute,
+            Nodes::Casted.new(3, attribute)
+          )
+        end
+
+        it 'can be constructed with an exclusive range starting from -Infinity' do
+          attribute = Attribute.new nil, nil
+          node = attribute.between(-::Float::INFINITY...3)
+
+          node.must_equal Nodes::LessThan.new(
+            attribute,
+            Nodes::Casted.new(3, attribute)
+          )
+        end
+
+        it 'can be constructed with an infinite range' do
+          attribute = Attribute.new nil, nil
+          node = attribute.between(-::Float::INFINITY..::Float::INFINITY)
+
+          node.must_equal Nodes::NotIn.new(attribute, [])
+        end
+
+        it 'can be constructed with a range ending at Infinity' do
+          attribute = Attribute.new nil, nil
+          node = attribute.between(0..::Float::INFINITY)
+
+          node.must_equal Nodes::GreaterThanOrEqual.new(
+            attribute,
+            Nodes::Casted.new(0, attribute)
+          )
+        end
+
+        it 'can be constructed with an exclusive range' do
+          attribute = Attribute.new nil, nil
+          node = attribute.between(0...3)
+
+          node.must_equal Nodes::And.new([
+            Nodes::GreaterThanOrEqual.new(
+              attribute,
+              Nodes::Casted.new(0, attribute)
+            ),
+            Nodes::LessThan.new(
+              attribute,
+              Nodes::Casted.new(3, attribute)
+            )
+          ])
+        end
+      end
+
+      describe '#in' do
+        it 'can be constructed with a subquery' do
+          relation = Table.new(:users)
+          mgr = relation.project relation[:id]
+          mgr.where relation[:name].does_not_match_all(['%chunky%','%bacon%'])
+          attribute = Attribute.new nil, nil
+
+          node = attribute.in(mgr)
+
+          node.must_equal Nodes::In.new(attribute, mgr.ast)
+        end
+
+        it 'can be constructed with a list' do
+          attribute = Attribute.new nil, nil
+          node = attribute.in([1, 2, 3])
+
+          node.must_equal Nodes::In.new(
+            attribute,
+            [
+              Nodes::Casted.new(1, attribute),
+              Nodes::Casted.new(2, attribute),
+              Nodes::Casted.new(3, attribute),
+            ]
+          )
+        end
+
+        it 'can be constructed with a random object' do
+          attribute = Attribute.new nil, nil
+          random_object = Object.new
+          node = attribute.in(random_object)
+
+          node.must_equal Nodes::In.new(
+            attribute,
+            Nodes::Casted.new(random_object, attribute)
+          )
         end
 
         it 'should generate IN in sql' do
@@ -549,15 +695,112 @@ module Arel
         end
       end
 
-      describe '#not_in' do
-        it 'can be constructed with a list' do
+      describe 'with a range' do
+        it 'can be constructed with a standard range' do
+          attribute = Attribute.new nil, nil
+          node = attribute.not_between(1..3)
+
+          node.must_equal Nodes::Grouping.new(Nodes::Or.new(
+            Nodes::LessThan.new(
+              attribute,
+              Nodes::Casted.new(1, attribute)
+            ),
+            Nodes::GreaterThan.new(
+              attribute,
+              Nodes::Casted.new(3, attribute)
+            )
+          ))
         end
 
-        it 'should return a NotIn node' do
+        it 'can be constructed with a range starting from -Infinity' do
           attribute = Attribute.new nil, nil
-          node = Nodes::NotIn.new attribute, [1,2,3]
-          node.left.must_equal attribute
-          node.right.must_equal [1, 2, 3]
+          node = attribute.not_between(-::Float::INFINITY..3)
+
+          node.must_equal Nodes::GreaterThan.new(
+            attribute,
+            Nodes::Casted.new(3, attribute)
+          )
+        end
+
+        it 'can be constructed with an exclusive range starting from -Infinity' do
+          attribute = Attribute.new nil, nil
+          node = attribute.not_between(-::Float::INFINITY...3)
+
+          node.must_equal Nodes::GreaterThanOrEqual.new(
+            attribute,
+            Nodes::Casted.new(3, attribute)
+          )
+        end
+
+        it 'can be constructed with an infinite range' do
+          attribute = Attribute.new nil, nil
+          node = attribute.not_between(-::Float::INFINITY..::Float::INFINITY)
+
+          node.must_equal Nodes::In.new(attribute, [])
+        end
+
+        it 'can be constructed with a range ending at Infinity' do
+          attribute = Attribute.new nil, nil
+          node = attribute.not_between(0..::Float::INFINITY)
+
+          node.must_equal Nodes::LessThan.new(
+            attribute,
+            Nodes::Casted.new(0, attribute)
+          )
+        end
+
+        it 'can be constructed with an exclusive range' do
+          attribute = Attribute.new nil, nil
+          node = attribute.not_between(0...3)
+
+          node.must_equal Nodes::Grouping.new(Nodes::Or.new(
+            Nodes::LessThan.new(
+              attribute,
+              Nodes::Casted.new(0, attribute)
+            ),
+            Nodes::GreaterThanOrEqual.new(
+              attribute,
+              Nodes::Casted.new(3, attribute)
+            )
+          ))
+        end
+      end
+
+      describe '#not_in' do
+        it 'can be constructed with a subquery' do
+          relation = Table.new(:users)
+          mgr = relation.project relation[:id]
+          mgr.where relation[:name].does_not_match_all(['%chunky%','%bacon%'])
+          attribute = Attribute.new nil, nil
+
+          node = attribute.not_in(mgr)
+
+          node.must_equal Nodes::NotIn.new(attribute, mgr.ast)
+        end
+
+        it 'can be constructed with a list' do
+          attribute = Attribute.new nil, nil
+          node = attribute.not_in([1, 2, 3])
+
+          node.must_equal Nodes::NotIn.new(
+            attribute,
+            [
+              Nodes::Casted.new(1, attribute),
+              Nodes::Casted.new(2, attribute),
+              Nodes::Casted.new(3, attribute),
+            ]
+          )
+        end
+
+        it 'can be constructed with a random object' do
+          attribute = Attribute.new nil, nil
+          random_object = Object.new
+          node = attribute.not_in(random_object)
+
+          node.must_equal Nodes::NotIn.new(
+            attribute,
+            Nodes::Casted.new(random_object, attribute)
+          )
         end
 
         it 'should generate NOT IN in sql' do
