@@ -10,13 +10,13 @@ module Arel
       def test_default_to_sql
         viz  = Arel::Visitors::ToSql.new Table.engine.connection_pool
         node = Arel::Nodes::Bin.new(Arel.sql('zomg'))
-        assert_equal 'zomg', viz.accept(node)
+        assert_equal 'zomg', viz.accept(node, Collectors::SQLString.new).value
       end
 
       def test_mysql_to_sql
         viz  = Arel::Visitors::MySQL.new Table.engine.connection_pool
         node = Arel::Nodes::Bin.new(Arel.sql('zomg'))
-        assert_equal 'BINARY zomg', viz.accept(node)
+        assert_equal 'BINARY zomg', viz.accept(node, Collectors::SQLString.new).value
       end
 
       def test_equality_with_same_ivars
