@@ -4,7 +4,7 @@ module Arel
   module Nodes
     describe 'Grouping' do
       it 'should create Equality nodes' do
-        grouping = Grouping.new('foo')
+        grouping = Grouping.new(Nodes.build_quoted('foo'))
         grouping.eq('foo').to_sql.must_be_like %q{('foo') = 'foo'}
       end
 
