@@ -5,7 +5,7 @@ module Arel
   describe 'select manager' do
     def test_join_sources
       manager = Arel::SelectManager.new Table.engine
-      manager.join_sources << Arel::Nodes::StringJoin.new('foo')
+      manager.join_sources << Arel::Nodes::StringJoin.new(Nodes.build_quoted('foo'))
       assert_equal "SELECT FROM 'foo'", manager.to_sql
     end
 
@@ -33,7 +33,7 @@ module Arel
         it 'accepts symbols' do
           table   = Table.new :users
           manager = Arel::SelectManager.new Table.engine
-          manager.project SqlLiteral.new '*'
+          manager.project Nodes::SqlLiteral.new '*'
           manager.from table
           manager.order :foo
           manager.to_sql.must_be_like %{ SELECT * FROM "users" ORDER BY foo }
@@ -140,7 +140,7 @@ module Arel
           mgr.to_sql.must_be_like %{ SELECT  FROM "users" INNER JOIN "users" "users_2" ON omg }
         end
 
-        it 'converts to sqlliterals' do
+        it 'converts to sqlliterals with multiple items' do
           table = Table.new :users
           right = table.alias
           mgr   = table.from table
@@ -152,7 +152,7 @@ module Arel
 
     describe 'clone' do
       it 'creates new cores' do
-        table   = Table.new :users, :engine => Table.engine, :as => 'foo'
+        table   = Table.new :users, :as => 'foo'
         mgr = table.from table
         m2 = mgr.clone
         m2.project "foo"
@@ -160,7 +160,7 @@ module Arel
       end
 
       it 'makes updates to the correct copy' do
-        table   = Table.new :users, :engine => Table.engine, :as => 'foo'
+        table   = Table.new :users, :as => 'foo'
         mgr = table.from table
         m2 = mgr.clone
         m3 = m2.clone
@@ -172,7 +172,7 @@ module Arel
 
     describe 'initialize' do
       it 'uses alias in sql' do
-        table   = Table.new :users, :engine => Table.engine, :as => 'foo'
+        table   = Table.new :users, :as => 'foo'
         mgr = table.from table
         mgr.skip 10
         mgr.to_sql.must_be_like %{ SELECT FROM "users" "foo" OFFSET 10 }
@@ -224,7 +224,7 @@ module Arel
       it 'should create an exists clause' do
         table = Table.new(:users)
         manager = Arel::SelectManager.new Table.engine, table
-        manager.project SqlLiteral.new '*'
+        manager.project Nodes::SqlLiteral.new '*'
         m2 = Arel::SelectManager.new(manager.engine)
         m2.project manager.exists
         m2.to_sql.must_be_like %{ SELECT EXISTS (#{manager.to_sql}) }
@@ -233,7 +233,7 @@ module Arel
       it 'can be aliased' do
         table = Table.new(:users)
         manager = Arel::SelectManager.new Table.engine, table
-        manager.project SqlLiteral.new '*'
+        manager.project Nodes::SqlLiteral.new '*'
         m2 = Arel::SelectManager.new(manager.engine)
         m2.project manager.exists.as('foo')
         m2.to_sql.must_be_like %{ SELECT EXISTS (#{manager.to_sql}) AS foo }
@@ -307,13 +307,11 @@ module Arel
         table = Table.new :users
         @m1 = Arel::SelectManager.new Table.engine, table
         @m1.project Arel.star
-        @m1.where(table[:age].in(18..60))
+        @m1.where(table[:age].between(18..60))
 
         @m2 = Arel::SelectManager.new Table.engine, table
         @m2.project Arel.star
-        @m2.where(table[:age].in(40..99))
-
-
+        @m2.where(table[:age].between(40..99))
       end
 
       it 'should except two managers' do
@@ -330,6 +328,20 @@ module Arel
     end
 
     describe 'with' do
+      it 'should support basic WITH' do
+        users          = Table.new(:users)
+        users_top      = Table.new(:users_top)
+        comments       = Table.new(:comments)
+
+        top            = users.project(users[:id]).where(users[:karma].gt(100))
+        users_as       = Arel::Nodes::As.new(users_top, top)
+        select_manager = comments.project(Arel.star).with(users_as)
+                          .where(comments[:author_id].in(users_top.project(users_top[:id])))
+
+        select_manager.to_sql.must_be_like %{
+          WITH "users_top" AS (SELECT "users"."id" FROM "users" WHERE "users"."karma" > 100) SELECT * FROM "comments" WHERE "comments"."author_id" IN (SELECT "users_top"."id" FROM "users_top")
+        }
+      end
 
       it "should support WITH RECURSIVE" do
         comments           = Table.new(:comments)
@@ -368,9 +380,9 @@ module Arel
       it 'should return the ast' do
         table   = Table.new :users
         mgr = table.from table
-        ast = mgr.ast
-        mgr.visitor.accept(ast).must_equal mgr.to_sql
+        assert mgr.ast
       end
+
       it 'should allow orders to work when the ast is grepped' do
         table   = Table.new :users
         mgr = table.from table
@@ -413,7 +425,7 @@ module Arel
       it 'generates order clauses' do
         table   = Table.new :users
         manager = Arel::SelectManager.new Table.engine
-        manager.project SqlLiteral.new '*'
+        manager.project Nodes::SqlLiteral.new '*'
         manager.from table
         manager.order table[:id]
         manager.to_sql.must_be_like %{
@@ -425,7 +437,7 @@ module Arel
       it 'takes *args' do
         table   = Table.new :users
         manager = Arel::SelectManager.new Table.engine
-        manager.project SqlLiteral.new '*'
+        manager.project Nodes::SqlLiteral.new '*'
         manager.from table
         manager.order table[:id], table[:name]
         manager.to_sql.must_be_like %{
@@ -442,25 +454,13 @@ module Arel
       it 'has order attributes' do
         table   = Table.new :users
         manager = Arel::SelectManager.new Table.engine
-        manager.project SqlLiteral.new '*'
+        manager.project Nodes::SqlLiteral.new '*'
         manager.from table
         manager.order table[:id].desc
         manager.to_sql.must_be_like %{
           SELECT * FROM "users" ORDER BY "users"."id" DESC
         }
       end
-
-      it 'has order attributes for expressions' do
-        table   = Table.new :users
-        manager = Arel::SelectManager.new Table.engine
-        manager.project SqlLiteral.new '*'
-        manager.from table
-        manager.order table[:id].count.desc
-        manager.to_sql.must_be_like %{
-          SELECT * FROM "users" ORDER BY COUNT("users"."id") DESC
-        }
-      end
-
     end
 
     describe 'on' do
@@ -529,7 +529,15 @@ module Arel
       assert_equal 'bar', join.right
     end
 
-    it 'should create join nodes with a klass' do
+    it 'should create join nodes with a full outer join klass' do
+      relation = Arel::SelectManager.new Table.engine
+      join = relation.create_join 'foo', 'bar', Arel::Nodes::FullOuterJoin
+      assert_kind_of Arel::Nodes::FullOuterJoin, join
+      assert_equal 'foo', join.left
+      assert_equal 'bar', join.right
+    end
+
+    it 'should create join nodes with a outer join klass' do
       relation = Arel::SelectManager.new Table.engine
       join = relation.create_join 'foo', 'bar', Arel::Nodes::OuterJoin
       assert_kind_of Arel::Nodes::OuterJoin, join
@@ -537,6 +545,14 @@ module Arel
       assert_equal 'bar', join.right
     end
 
+    it 'should create join nodes with a right outer join klass' do
+      relation = Arel::SelectManager.new Table.engine
+      join = relation.create_join 'foo', 'bar', Arel::Nodes::RightOuterJoin
+      assert_kind_of Arel::Nodes::RightOuterJoin, join
+      assert_equal 'foo', join.left
+      assert_equal 'bar', join.right
+    end
+
     describe 'join' do
       it 'responds to join' do
         left      = Table.new :users
@@ -568,21 +584,73 @@ module Arel
         }
       end
 
+      it 'takes the full outer join class' do
+        left      = Table.new :users
+        right     = left.alias
+        predicate = left[:id].eq(right[:id])
+        manager   = Arel::SelectManager.new Table.engine
+
+        manager.from left
+        manager.join(right, Nodes::FullOuterJoin).on(predicate)
+        manager.to_sql.must_be_like %{
+           SELECT FROM "users"
+             FULL OUTER JOIN "users" "users_2"
+               ON "users"."id" = "users_2"."id"
+        }
+      end
+
+      it 'takes the right outer join class' do
+        left      = Table.new :users
+        right     = left.alias
+        predicate = left[:id].eq(right[:id])
+        manager   = Arel::SelectManager.new Table.engine
+
+        manager.from left
+        manager.join(right, Nodes::RightOuterJoin).on(predicate)
+        manager.to_sql.must_be_like %{
+           SELECT FROM "users"
+             RIGHT OUTER JOIN "users" "users_2"
+               ON "users"."id" = "users_2"."id"
+        }
+      end
+
       it 'noops on nil' do
         manager   = Arel::SelectManager.new Table.engine
         manager.join(nil).must_equal manager
       end
     end
 
+    describe 'outer join' do
+      it 'responds to join' do
+        left      = Table.new :users
+        right     = left.alias
+        predicate = left[:id].eq(right[:id])
+        manager   = Arel::SelectManager.new Table.engine
+
+        manager.from left
+        manager.outer_join(right).on(predicate)
+        manager.to_sql.must_be_like %{
+           SELECT FROM "users"
+             LEFT OUTER JOIN "users" "users_2"
+               ON "users"."id" = "users_2"."id"
+        }
+      end
+
+      it 'noops on nil' do
+        manager   = Arel::SelectManager.new Table.engine
+        manager.outer_join(nil).must_equal manager
+      end
+    end
+
     describe 'joins' do
-      it 'returns join sql' do
+
+      it 'returns inner join sql' do
         table   = Table.new :users
         aliaz   = table.alias
         manager = Arel::SelectManager.new Table.engine
         manager.from Nodes::InnerJoin.new(aliaz, table[:id].eq(aliaz[:id]))
-        manager.join_sql.must_be_like %{
-          INNER JOIN "users" "users_2" "users"."id" = "users_2"."id"
-        }
+        assert_match 'INNER JOIN "users" "users_2" "users"."id" = "users_2"."id"',
+                     manager.to_sql
       end
 
       it 'returns outer join sql' do
@@ -590,9 +658,8 @@ module Arel
         aliaz   = table.alias
         manager = Arel::SelectManager.new Table.engine
         manager.from Nodes::OuterJoin.new(aliaz, table[:id].eq(aliaz[:id]))
-        manager.join_sql.must_be_like %{
-          LEFT OUTER JOIN "users" "users_2" "users"."id" = "users_2"."id"
-        }
+        assert_match 'LEFT OUTER JOIN "users" "users_2" "users"."id" = "users_2"."id"',
+                     manager.to_sql
       end
 
       it 'can have a non-table alias as relation name' do
@@ -612,25 +679,26 @@ module Arel
         }
       end
 
+      it "joins itself" do
+        left      = Table.new :users
+        right     = left.alias
+        predicate = left[:id].eq(right[:id])
+
+        mgr = left.join(right)
+        mgr.project Nodes::SqlLiteral.new('*')
+        mgr.on(predicate).must_equal mgr
+
+        mgr.to_sql.must_be_like %{
+           SELECT * FROM "users"
+             INNER JOIN "users" "users_2"
+               ON "users"."id" = "users_2"."id"
+        }
+      end
+
       it 'returns string join sql' do
         manager = Arel::SelectManager.new Table.engine
-        manager.from Nodes::StringJoin.new('hello')
-        manager.join_sql.must_be_like %{ 'hello' }
-      end
-
-      it 'returns nil join sql' do
-        manager = Arel::SelectManager.new Table.engine
-        manager.join_sql.must_be_nil
-      end
-    end
-
-    describe 'order_clauses' do
-      it 'returns order clauses as a list' do
-        table   = Table.new :users
-        manager = Arel::SelectManager.new Table.engine
-        manager.from table
-        manager.order table[:id]
-        manager.order_clauses.first.must_be_like %{ "users"."id" }
+        manager.from Nodes::StringJoin.new(Nodes.build_quoted('hello'))
+        assert_match "'hello'", manager.to_sql
       end
     end
 
@@ -692,6 +760,47 @@ module Arel
         }
       end
 
+      it 'takes an order with multiple columns' do
+        table   = Table.new :users
+        manager = Arel::SelectManager.new Table.engine
+        manager.from table
+        manager.window('a_window').order(table['foo'].asc, table['bar'].desc)
+        manager.to_sql.must_be_like %{
+          SELECT FROM "users" WINDOW "a_window" AS (ORDER BY "users"."foo" ASC, "users"."bar" DESC)
+        }
+      end
+
+      it 'takes a partition' do
+        table   = Table.new :users
+        manager = Arel::SelectManager.new Table.engine
+        manager.from table
+        manager.window('a_window').partition(table['bar'])
+        manager.to_sql.must_be_like %{
+          SELECT FROM "users" WINDOW "a_window" AS (PARTITION BY "users"."bar")
+        }
+      end
+
+      it 'takes a partition and an order' do
+        table   = Table.new :users
+        manager = Arel::SelectManager.new Table.engine
+        manager.from table
+        manager.window('a_window').partition(table['foo']).order(table['foo'].asc)
+        manager.to_sql.must_be_like %{
+          SELECT FROM "users" WINDOW "a_window" AS (PARTITION BY "users"."foo"
+            ORDER BY "users"."foo" ASC)
+        }
+      end
+
+      it 'takes a partition with multiple columns' do
+        table   = Table.new :users
+        manager = Arel::SelectManager.new Table.engine
+        manager.from table
+        manager.window('a_window').partition(table['bar'], table['baz'])
+        manager.to_sql.must_be_like %{
+          SELECT FROM "users" WINDOW "a_window" AS (PARTITION BY "users"."bar", "users"."baz")
+        }
+      end
+
       it 'takes a rows frame, unbounded preceding' do
         table   = Table.new :users
         manager = Arel::SelectManager.new Table.engine
@@ -868,12 +977,33 @@ module Arel
     end
 
     describe 'update' do
+
+      it 'creates an update statement' do
+        table   = Table.new :users
+        manager = Arel::SelectManager.new Table.engine
+        manager.from table
+        stmt = manager.compile_update({table[:id] => 1}, Arel::Attributes::Attribute.new(table, 'id'))
+
+        stmt.to_sql.must_be_like %{
+          UPDATE "users" SET "id" = 1
+        }
+      end
+
+      it 'takes a string' do
+        table   = Table.new :users
+        manager = Arel::SelectManager.new Table.engine
+        manager.from table
+        stmt = manager.compile_update(Nodes::SqlLiteral.new('foo = bar'), Arel::Attributes::Attribute.new(table, 'id'))
+
+        stmt.to_sql.must_be_like %{ UPDATE "users" SET foo = bar }
+      end
+
       it 'copies limits' do
         table   = Table.new :users
         manager = Arel::SelectManager.new Table.engine
         manager.from table
         manager.take 1
-        stmt = manager.compile_update(SqlLiteral.new('foo = bar'), Arel::Attributes::Attribute.new(table, 'id'))
+        stmt = manager.compile_update(Nodes::SqlLiteral.new('foo = bar'), Arel::Attributes::Attribute.new(table, 'id'))
         stmt.key = table['id']
 
         stmt.to_sql.must_be_like %{
@@ -887,7 +1017,7 @@ module Arel
         manager = Arel::SelectManager.new Table.engine
         manager.from table
         manager.order :foo
-        stmt = manager.compile_update(SqlLiteral.new('foo = bar'), Arel::Attributes::Attribute.new(table, 'id'))
+        stmt = manager.compile_update(Nodes::SqlLiteral.new('foo = bar'), Arel::Attributes::Attribute.new(table, 'id'))
         stmt.key = table['id']
 
         stmt.to_sql.must_be_like %{
@@ -896,15 +1026,6 @@ module Arel
         }
       end
 
-      it 'takes a string' do
-        table   = Table.new :users
-        manager = Arel::SelectManager.new Table.engine
-        manager.from table
-        stmt = manager.compile_update(SqlLiteral.new('foo = bar'), Arel::Attributes::Attribute.new(table, 'id'))
-
-        stmt.to_sql.must_be_like %{ UPDATE "users" SET foo = bar }
-      end
-
       it 'copies where clauses' do
         table   = Table.new :users
         manager = Arel::SelectManager.new Table.engine
@@ -930,19 +1051,15 @@ module Arel
         }
       end
 
-      it 'executes an update statement' do
-        table   = Table.new :users
-        manager = Arel::SelectManager.new Table.engine
-        manager.from table
-        stmt = manager.compile_update({table[:id] => 1}, Arel::Attributes::Attribute.new(table, 'id'))
-
-        stmt.to_sql.must_be_like %{
-          UPDATE "users" SET "id" = 1
-        }
-      end
     end
 
     describe 'project' do
+      it "takes sql literals" do
+        manager = Arel::SelectManager.new Table.engine
+        manager.project Nodes::SqlLiteral.new '*'
+        manager.to_sql.must_be_like %{ SELECT * }
+      end
+
       it 'takes multiple args' do
         manager = Arel::SelectManager.new Table.engine
         manager.project Nodes::SqlLiteral.new('foo'),
@@ -956,11 +1073,6 @@ module Arel
         manager.to_sql.must_be_like %{ SELECT * }
       end
 
-      it "takes sql literals" do
-        manager = Arel::SelectManager.new Table.engine
-        manager.project Nodes::SqlLiteral.new '*'
-        manager.to_sql.must_be_like %{ SELECT * }
-      end
     end
 
     describe 'projections' do
@@ -1032,24 +1144,6 @@ module Arel
       end
     end
 
-    describe "join" do
-      it "joins itself" do
-        left      = Table.new :users
-        right     = left.alias
-        predicate = left[:id].eq(right[:id])
-
-        mgr = left.join(right)
-        mgr.project Nodes::SqlLiteral.new('*')
-        mgr.on(predicate).must_equal mgr
-
-        mgr.to_sql.must_be_like %{
-           SELECT * FROM "users"
-             INNER JOIN "users" "users_2"
-               ON "users"."id" = "users_2"."id"
-        }
-      end
-    end
-
     describe 'from' do
       it "makes sql" do
         table   = Table.new :users
@@ -1085,6 +1179,33 @@ module Arel
         manager.distinct(false)
         manager.ast.cores.last.set_quantifier.must_equal nil
       end
+
+      it "chains" do
+        manager = Arel::SelectManager.new Table.engine
+        manager.distinct.must_equal manager
+        manager.distinct(false).must_equal manager
+      end
+    end
+
+    describe 'distinct_on' do
+      it 'sets the quantifier' do
+        manager = Arel::SelectManager.new Table.engine
+        table = Table.new :users
+
+        manager.distinct_on(table['id'])
+        manager.ast.cores.last.set_quantifier.must_equal Arel::Nodes::DistinctOn.new(table['id'])
+
+        manager.distinct_on(false)
+        manager.ast.cores.last.set_quantifier.must_equal nil
+      end
+
+      it "chains" do
+        manager = Arel::SelectManager.new Table.engine
+        table = Table.new :users
+
+        manager.distinct_on(table['id']).must_equal manager
+        manager.distinct_on(false).must_equal manager
+      end
     end
   end
 end
