@@ -19,6 +19,13 @@ module Arel
       assert_equal 'bar', join.right
     end
 
+    it 'should create join nodes with a klass' do
+      join = @relation.create_join 'foo', 'bar', Arel::Nodes::FullOuterJoin
+      assert_kind_of Arel::Nodes::FullOuterJoin, join
+      assert_equal 'foo', join.left
+      assert_equal 'bar', join.right
+    end
+
     it 'should create join nodes with a klass' do
       join = @relation.create_join 'foo', 'bar', Arel::Nodes::OuterJoin
       assert_kind_of Arel::Nodes::OuterJoin, join
@@ -26,10 +33,18 @@ module Arel
       assert_equal 'bar', join.right
     end
 
+    it 'should create join nodes with a klass' do
+      join = @relation.create_join 'foo', 'bar', Arel::Nodes::RightOuterJoin
+      assert_kind_of Arel::Nodes::RightOuterJoin, join
+      assert_equal 'foo', join.left
+      assert_equal 'bar', join.right
+    end
+
     it 'should return an insert manager' do
       im = @relation.compile_insert 'VALUES(NULL)'
       assert_kind_of Arel::InsertManager, im
-      assert_equal 'INSERT INTO NULL VALUES(NULL)', im.to_sql
+      im.into Table.new(:users)
+      assert_equal "INSERT INTO \"users\" VALUES(NULL)", im.to_sql
     end
 
     it 'should return IM from insert_manager' do
@@ -52,6 +67,22 @@ module Arel
       end
     end
 
+    describe 'update_manager' do
+      it 'should return an update manager' do
+        um = @relation.update_manager
+        assert_kind_of Arel::UpdateManager, um
+        assert_equal um.engine, @relation.engine
+      end
+    end
+
+    describe 'delete_manager' do
+      it 'should return a delete manager' do
+        dm = @relation.delete_manager
+        assert_kind_of Arel::DeleteManager, dm
+        assert_equal dm.engine, @relation.engine
+      end
+    end
+
     describe 'having' do
       it 'adds a having clause' do
         mgr = @relation.having @relation[:id].eq(10)
@@ -81,6 +112,20 @@ module Arel
           }
         end
       end
+
+      describe 'join' do
+        it 'creates an outer join' do
+          right     = @relation.alias
+          predicate = @relation[:id].eq(right[:id])
+          mgr = @relation.outer_join(right).on(predicate)
+
+          mgr.to_sql.must_be_like %{
+            SELECT FROM "users"
+              LEFT OUTER JOIN "users" "users_2"
+                ON "users"."id" = "users_2"."id"
+          }
+        end
+      end
     end
 
     describe 'group' do
@@ -130,19 +175,19 @@ module Arel
     describe 'take' do
       it "should add a limit" do
         manager = @relation.take 1
-        manager.project SqlLiteral.new '*'
+        manager.project Nodes::SqlLiteral.new '*'
         manager.to_sql.must_be_like %{ SELECT * FROM "users" LIMIT 1 }
       end
     end
 
     describe 'project' do
       it 'can project' do
-        manager = @relation.project SqlLiteral.new '*'
+        manager = @relation.project Nodes::SqlLiteral.new '*'
         manager.to_sql.must_be_like %{ SELECT * FROM "users" }
       end
 
       it 'takes multiple parameters' do
-        manager = @relation.project SqlLiteral.new('*'), SqlLiteral.new('*')
+        manager = @relation.project Nodes::SqlLiteral.new('*'), Nodes::SqlLiteral.new('*')
         manager.to_sql.must_be_like %{ SELECT *, * FROM "users" }
       end
     end
