@@ -1,4 +1,5 @@
 require 'helper'
+require 'set'
 
 module Arel
   module Visitors
@@ -81,12 +82,24 @@ module Arel
         assert_equal [:a, :b, join], @collector.calls
       end
 
+      def test_full_outer_join
+        join = Nodes::FullOuterJoin.new :a, :b
+        @visitor.accept join
+        assert_equal [:a, :b, join], @collector.calls
+      end
+
       def test_outer_join
         join = Nodes::OuterJoin.new :a, :b
         @visitor.accept join
         assert_equal [:a, :b, join], @collector.calls
       end
 
+      def test_right_outer_join
+        join = Nodes::RightOuterJoin.new :a, :b
+        @visitor.accept join
+        assert_equal [:a, :b, join], @collector.calls
+      end
+
       [
         Arel::Nodes::Assignment,
         Arel::Nodes::Between,
@@ -159,6 +172,13 @@ module Arel
         assert_equal [:a, :b, node, list], @collector.calls
       end
 
+      def test_set
+        node = Nodes::Or.new(:a, :b)
+        set  = Set.new([node])
+        @visitor.accept set
+        assert_equal [:a, :b, node, set], @collector.calls
+      end
+
       def test_hash
         node = Nodes::Or.new(:a, :b)
         hash = { node => node }
